package org.codehaus.plexus.languages.java.jpms;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/**
 * Holds the results of the location manager
 * 
 * @author Robert Scholte
 * @since 1.0.0
 */
public class ResolvePathResult
{
    private JavaModuleDescriptor moduleDescriptor;
    
    private ModuleNameSource moduleNameSource;
    
    ResolvePathResult()
    {
    }

    ResolvePathResult setModuleDescriptor( JavaModuleDescriptor moduleDescriptor )
    {
        this.moduleDescriptor = moduleDescriptor;
        return this;
    }
    
    public JavaModuleDescriptor getModuleDescriptor()
    {
        return moduleDescriptor;
    }
    
    ResolvePathResult setModuleNameSource( ModuleNameSource moduleNameSource )
    {
        this.moduleNameSource = moduleNameSource;
        return this;
    }
    
    public ModuleNameSource getModuleNameSource()
    {
        return moduleNameSource;
    }
}
